STATE_ACTIVE = (True, 2)
TRUE_VALUES = ["+", "Ja", "yes", "Yes", "ja"]
FALSE_VALUES = ["-", "Nein", "no", "No", "nein"]
SEXES = {
    "w": "f",
    "m": "m",
    "weiblich": "f",
    "männlich": "m",
    "female": "f",
    "male": "m",
}
