# Generated by Django 3.2.10 on 2021-12-30 10:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0019_fix_uniqueness_per_site'),
        ('csv_import', '0003_fix_uniqueness_per_site'),
    ]

    operations = [
        migrations.AddField(
            model_name='importtemplatefield',
            name='args',
            field=models.JSONField(default={}, verbose_name='Optional arguments passed to field type'),
        ),
    ]
