# Generated by Django 3.2.10 on 2022-01-01 22:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('csv_import', '0004_args'),
    ]

    operations = [
        migrations.AddField(
            model_name='importtemplate',
            name='has_index_col',
            field=models.BooleanField(default=False, verbose_name='Has the CSV file an own index column?'),
        ),
    ]
