# Generated by Django 3.2.11 on 2022-01-07 16:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('csv_import', '0005_optional_index_col'),
    ]

    operations = [
        migrations.AddField(
            model_name='importtemplate',
            name='from_default',
            field=models.BooleanField(default=False),
        ),
    ]
