# Generated by Django 3.2.11 on 2022-01-29 22:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('csv_import', '0007_virtual_template_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='importtemplate',
            name='quotechar',
            field=models.CharField(default='"', max_length=255, verbose_name='CSV quote character'),
        ),
    ]
