{# -*- engine:django -*- #}

{% extends "core/base.html" %}
{% load material_form i18n static %}

{% block browser_title %}{% trans "Upload import template" %}{% endblock %}
{% block page_title %}{% trans "Upload import template" %}{% endblock %}

{% block content %}
  <form method="post" enctype="multipart/form-data">
    {% csrf_token %}
    {% form form=form %}{% endform %}

    {% include "core/partials/save_button.html" with icon="cloud_upload" caption=_("Upload import template") %}
  </form>
{% endblock %}
