# Generated by Django 3.2.11 on 2022-01-28 19:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('csv_import', '0006_from_default_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='importtemplatefield',
            name='virtual_tmpl',
            field=models.TextField(blank=True, null=True, verbose_name='Django template to generate virtual field from'),
        ),
        migrations.AddField(
            model_name='importtemplatefield',
            name='virtual',
            field=models.BooleanField(default=False, verbose_name='Virtual field'),
        ),
    ]
